@@ -1,4 +1,5 @@
 use crate::*;
+use std::collections::HashSet;
 
 /// Derivative functions for `f32`s.
 pub mod f32;
@@ -36,9 +37,12 @@ pub use self::u64::*;
 /// Derivative functions for `u128`s.
 pub mod u128;
 pub use self::u128::*;
+// /// Derivative functions for [ndarray](https://docs.rs/ndarray/latest/ndarray/index.html).
+// pub mod ndarray;
+// pub use self::ndarray::*;
 
 /// Forward General Derivative type
-pub type FgdType = fn(&[String], String, &[Arg]) -> syn::Stmt;
+pub type FgdType = fn(String, &[Arg], &[String], &mut HashSet<String>) -> syn::Stmt;
 /// Reverse General Derivative type
 pub type RgdType = fn(String, &[Arg], &mut HashMap<String, Vec<String>>) -> syn::Stmt;
 
@@ -218,63 +222,74 @@ impl TryFrom<&str> for Type {
 /// forward_derivative_macro!(outer_test,"0f32","{0}-{1}","{0}*{1}+{0}");
 /// ```
 pub fn fgd<const DEFAULT: &'static str, const TRANSLATION_FUNCTIONS: &'static [DFn]>(
-    outer_fn_args: &[String],
     local_ident: String,
     args: &[Arg],
+    outer_fn_args: &[String],
+    non_zero_derivatives: &mut HashSet<String>,
 ) -> syn::Stmt {
-    assert_eq!(args.len(), TRANSLATION_FUNCTIONS.len());
+    assert_eq!(
+        args.len(),
+        TRANSLATION_FUNCTIONS.len(),
+        "fgd args len mismatch"
+    );
 
-    // Gets vec of deriative idents and derivative functions
-    let (idents, deriatives) = outer_fn_args
+    // Gets vec of derivative idents and derivative functions
+    let (idents, derivatives) = outer_fn_args
         .iter()
-        .map(|outer_fn_input| {
+        .filter_map(|outer_fn_input| {
             let acc = args
                 .iter()
                 .zip(TRANSLATION_FUNCTIONS.iter())
-                .map(|(arg,t)|
+                .filter_map(|(arg,t)|
                 // See the docs for cumulative (these if's accomplish the same-ish thing)
                 // TODO Improve docs here directly
                 match arg {
-                    Arg::Literal(_) => DEFAULT.to_string(),
+                    Arg::Literal(_) => None, // Since we are multiplying by `DEFAULT` (e.g. `0.`) we can simply ignore this property
                     Arg::Variable(v) => {
-                        let (a,b) = (
-                            t(args),
-                        if v == outer_fn_input {
-                            der!(outer_fn_input)
+                        let a = t(args);
+                        let b = if v == outer_fn_input {
+                            Some(der!(outer_fn_input))
                         } else if outer_fn_args.contains(v) {
-                            DEFAULT.to_string()
+                            None // Since we are multiplying by `DEFAULT` (e.g. `0.`) we can simply ignore this property
                         } else {
-                            wrt!(arg,outer_fn_input)
-                        });
+                            let der = wrt!(arg,outer_fn_input);
+                            // If the derivative has not been defined, we know it would've been defined as zero
+                            non_zero_derivatives.get(&der).cloned()
+                        };
                         // eprintln!("a: {}, b: {}",a,b);
-                        format!("({})*{}",a,b)
+                        match b {
+                            Some(acc_der) => Some(format!("({})*{}",a,acc_der)),
+                            None => None
+                        }
                     }
                 })
                 .intersperse(String::from("+"))
                 .collect::<String>();
-
-            (wrt!(local_ident, outer_fn_input), acc)
+            match acc.is_empty() {
+                true => None,
+                false => {
+                    let new_der = wrt!(local_ident, outer_fn_input);
+                    // If there are some non-zero components this derivative may be non-zero and is thus worth defining
+                    non_zero_derivatives.insert(new_der.clone());
+                    Some((new_der, acc))
+                }
+            }
         })
         .unzip::<_, _, Vec<_>, Vec<_>>();
-    // eprintln!("idents: {:?}",idents);
-    // eprintln!("deriatives: {:?}",deriatives);
 
     // Converts vec's to strings
-    let (idents, deriatives) = (
+    let (idents, derivatives) = (
         idents
             .into_iter()
             .intersperse(String::from(","))
             .collect::<String>(),
-        deriatives
+        derivatives
             .into_iter()
             .intersperse(String::from(","))
             .collect::<String>(),
     );
-    // eprintln!("idents: {}",idents);
-    // eprintln!("deriatives: {}",deriatives);
 
-    let stmt_str = format!("let ({}) = ({});", idents, deriatives);
-    // eprintln!("stmt_str: {}",stmt_str);
+    let stmt_str = format!("let ({}) = ({});", idents, derivatives);
     syn::parse_str(&stmt_str).expect("fgd: parse fail")
 }
 
@@ -284,9 +299,13 @@ pub fn rgd<const DEFAULT: &'static str, const TRANSLATION_FUNCTIONS: &'static [D
     args: &[Arg],
     component_map: &mut HashMap<String, Vec<String>>,
 ) -> syn::Stmt {
-    assert_eq!(args.len(), TRANSLATION_FUNCTIONS.len());
+    assert_eq!(
+        args.len(),
+        TRANSLATION_FUNCTIONS.len(),
+        "rgd args len mismatch"
+    );
 
-    let (idents, deriatives) = args
+    let (idents, derivatives) = args
         .iter()
         .zip(TRANSLATION_FUNCTIONS.iter())
         .filter_map(|(arg, t)| match arg {
@@ -303,18 +322,17 @@ pub fn rgd<const DEFAULT: &'static str, const TRANSLATION_FUNCTIONS: &'static [D
         })
         .unzip::<_, _, Vec<_>, Vec<_>>();
 
-    let (idents, deriatives) = (
+    let (idents, derivatives) = (
         idents
             .into_iter()
             .intersperse(String::from(","))
             .collect::<String>(),
-        deriatives
+        derivatives
             .into_iter()
             .intersperse(String::from(","))
             .collect::<String>(),
     );
 
-    let stmt_str = format!("let ({}) = ({});", idents, deriatives);
-    // eprintln!("stmt_str: {}", stmt_str);
+    let stmt_str = format!("let ({}) = ({});", idents, derivatives);
     syn::parse_str(&stmt_str).expect("fgd: parse fail")
 }
